/* atcat --- a member of the kat family (Keith's at package) */

/* atcat(1) displays the commands in a job queued by at(1).  Options are */
/* provided to display only the user entered commands or the entire job. */

/* Copyright (c) 1990, W. Keith Pyle, Austin, Texas */

#include "at.h"

char *verstring = PATCHLEVEL;

extern char *sys_errlist[];
extern char *optarg;

extern int opterr;
extern int optind;

/* -------------------------------------------------------------------------- */

main(argc, argv)

int argc;
char *argv[];

{
	char at_allow[MAXPATHLEN];
	char at_deny[MAXPATHLEN];
	char owner[16];

	int flag;
	int i;
	int show_no;
	int rc;
	int show_all;

	DIR *dp;

	show_all = FALSE;

	/* Get the user name */

	whoami(owner);

	/* Change to the at spool directory */

	if (chdir(ATDIR) < 0) {

		perror("can't change directory to the at directory");
		exit(1);
	}

	/* Check for allowed at usage */
	 
	if (!permit(owner, ALLOW_FILE, DENY_FILE)) {
 
		fprintf(stderr, "you are not authorized to use at.  Sorry.\n");
		exit(1);
	}

	/* Check for at least one argument */

	if (argc == 1) {

		fprintf(stderr, "usage: atcat [-e] job-number\n");
		exit(2);
	}

	/* Process any option flags found */

	while ((flag = getopt(argc, argv, "e")) != EOF) {

		switch (flag) {

			case 'e':

				show_all = TRUE;
				break;
			
			default:

				fprintf(stderr, "usage: atcat [-e] job-number\n");
				exit(2);
		}
	}

	/* Open the spool directory */
 
	if ((dp = opendir(".")) == NULL) {

		perror("can't read the at directory");
		exit(1);
	}

	/* Loop through all remaining arguments */

	rc = 0;

	for ( ; optind < argc ; optind++) {

		int jobno;

		/* Make sure we start at the beginning of the directory for each */

		rewinddir(dp);

		/* Try converting the argument to a number */

		show_no = atoi(argv[optind]);

		/* If the conversion didn't succeed (0 can't be a job no.), */
		/* then assume we have a user name and delete all entries for user */

		if (show_no == 0)
			rc |= find_all(argv[optind], show_all, dp);
		
		/* else, just delete the specified job number */

		else
			rc |= find_one(show_no, show_all, dp);
	}

	/* Close the spool directory and say bye */

	closedir(dp);
	exit(rc);
}

/* -------------------------------------------------------------------------- */

/* display -- display all or part of a job file */

display(name, jobno, show_all)

char *name;
int jobno;
int show_all;

{
	char line[MAXLINE];

	int rc;
	int uid;

	FILE *fp;

	if ((fp = fopen(name, "r")) == NULL) {

		sprintf(line, "can't open jobfile %d", jobno);
		perror(line);
		return(1);
	}

	if (!show_all) {

		while (fgets(line, sizeof(line), fp)) {

			if (strncmp(line, "umask ", 6) == 0)
				break;
		}
	}

	while (fgets(line, sizeof(line), fp))
		fputs(line, stdout);
	
	return(0);
}

/* -------------------------------------------------------------------------- */

find_one(show_no, show_all, dp)

int show_no;
int show_all;
DIR *dp;

{
	int jobno;
	int tod;

	struct dirent *entry;

	/* Scan the directory for one entry which matches the jobno */

	while ((entry = readdir(dp)) != NULL) {

		if (sscanf(entry->d_name, "%d.%d", &tod, &jobno) != 2)
			continue;

		if (jobno == show_no)
			return(display(entry->d_name, jobno, show_all));
	}

	fprintf(stderr, "atcat: %d: no such job number\n", show_no);

	return(1);
}

/* -------------------------------------------------------------------------- */

find_all(name, show_all, dp)

char *name;
int show_all;
DIR *dp;

{
	int jobno;
	int rc;
	int tod;
	int uid;

	struct dirent *entry;
	struct stat statbuf;

	rc = 0;

	/* If the owner name is "-", then delete all jobs for this user. */
	/* If the user is root, delete all jobs. */

	if (strcmp(name, "-") == 0) {

		uid = geteuid();

		if (uid == 0)
			uid = -1;		/* A flag meaning all jobs match */
	}

	else {

		struct passwd *pw;

		/* Get the uid of the named owner */

		if ((pw = getpwnam(name)) == NULL) {

			fprintf(stderr, "atrm: can't locate %s in /etc/passwd: %s\n",
				name, sys_errlist[errno]);
			return(1);
		}

		uid = pw->pw_uid;
	}

	/* Scan the directory for all job files with this owner */

	while ((entry = readdir(dp)) != NULL) {

		if (sscanf(entry->d_name, "%d.%d", &tod, &jobno) != 2)
			continue;

		if (uid >= 0) {

			if (stat(entry->d_name, &statbuf) < 0) {

				perror(entry->d_name);
				continue;
			}

			if (statbuf.st_uid != uid)
				continue;
		}

		rc |= display(entry->d_name, jobno, show_all);
	}

	return(rc);
}
