/* atrm --- a member of the kat family (Keith's at package) */

/* atrm(1) removes jobs queued by at(1) from the queue.  Options are */
/* provided to remove specific jobs or all of those by specified users. */

/* Copyright (c) 1990, W. Keith Pyle, Austin, Texas */

#include "at.h"

char *verstring = PATCHLEVEL;

extern char *sys_errlist[];
extern char *optarg;

extern int opterr;
extern int optind;

/* -------------------------------------------------------------------------- */

main(argc, argv)

int argc;
char *argv[];

{
	char at_allow[MAXPATHLEN];
	char at_deny[MAXPATHLEN];
	char owner[16];

	int flag;
	int force;
	int i;
	int interact;
	int killno;
	int rc;

	DIR *dp;

	/* Some initialization */

	force = FALSE;
	interact = FALSE;

	/* Get the user name */

	whoami(owner);

	/* Change to the at spool directory */

	if (chdir(ATDIR) < 0) {

		perror("can't change directory to the at directory");
		exit(1);
	}

	/* Check for allowed at usage */
	 
	if (!permit(owner, ALLOW_FILE, DENY_FILE)) {
 
		fprintf(stderr, "you are not authorized to use at.  Sorry.\n");
		exit(1);
	}

	/* Check for at least one argument */

	if (argc == 1) {

		fprintf(stderr, "usage: atrm [job-number] ... [username] ...\n");
		exit(2);
	}

	/* Process any option flags found */

	while ((flag = getopt(argc, argv, "fi")) != EOF) {

		switch (flag) {

			case 'f':

				force = TRUE;
				break;
			
			case 'i':

				interact = TRUE;
				break;
			
			default:

				fprintf(stderr,
					"usage: atrm [job-number] ... [username] ...\n");
				exit(2);
		}
	}

	/* If interaction was specified, turn off force and make sure stdin is */
	/* from a terminal device */

	if (interact) {

		force = FALSE;

		if (isatty(0) == 0)
			interact = FALSE;
	}

	/* Open the spool directory */
 
	if ((dp = opendir(".")) == NULL) {

		perror("can't read the at directory");
		exit(1);
	}

	/* Loop through all remaining arguments */

	rc = 0;

	for ( ; optind < argc ; optind++) {

		int jobno;

		/* Make sure we start at the beginning of the directory for each */

		rewinddir(dp);

		/* Try converting the argument to a number */

		killno = atoi(argv[optind]);

		/* If the conversion didn't succeed (0 can't be a job no.), */
		/* then assume we have a user name and delete all entries for user */

		if (killno == 0)
			rc |= find_all(argv[optind], force, interact, dp);
		
		/* else, just delete the specified job number */

		else
			rc |= find_one(killno, force, interact, dp);
	}

	/* Close the spool directory and say bye */

	closedir(dp);
	exit(rc);
}

/* -------------------------------------------------------------------------- */

/* delete --- handle all of the actual deletion work after an entry has been */
/* identified */

delete(name, jobno, force, interact)

char *name;
int jobno;
int force;
int interact;

{
	int rc;
	int uid;

	struct stat statbuf;

	/* Get some info on the job file */

	if (stat(name, &statbuf) < 0) {

		fprintf(stderr, "atrm: couldn't stat job file %s: %s\n",
			name, sys_errlist[errno]);
		return(1);
	}

	if (!force)
		printf("%6d: ", jobno);
	
	/* If this isn't being done by root, make sure the user owns the job */

	uid = geteuid();

	if (uid != 0) {
	
		if (statbuf.st_uid != uid) {

			if (!force)
				printf("permission denied\n", jobno);
			return(1);
		}
	}

	/* Ick, we have to talk to a user */

	if (interact) {

		char response[80];

		/* Is root blasting someone else's job? */

		if (uid == 0 && statbuf.st_uid != uid) {

			struct passwd *pw;

			if ((pw = getpwuid(statbuf.st_uid)) == NULL)
				fprintf(stderr,
					"atrm: can't locate uid %d in /etc/passwd: %s\n",
					uid, sys_errlist[errno]);
			else
				printf("(owned by %s) ", pw->pw_name);
		}

		printf("remove it? ");

		if (fgets(response, sizeof(response), stdin) == NULL)
			exit(1);
		
		if (response[0] != 'y' && response[0] != 'Y')
			return(0);
	}

	/* Try to remove the job file */

	rc = unlink(name);

	/* Report on the results, if appropriate */

	if (!force && !interact) {

		if (rc == 0)
			printf("removed\n", jobno);
		else
			printf("could not remove\n");
	}

	return(rc != 0);
}

/* -------------------------------------------------------------------------- */

find_one(killno, force, interact, dp)

int killno;
int force;
int interact;
DIR *dp;

{
	int jobno;
	int tod;

	struct dirent *entry;

	/* Scan the directory for one entry which matches the jobno */

	while ((entry = readdir(dp)) != NULL) {

		if (sscanf(entry->d_name, "%d.%d", &tod, &jobno) != 2)
			continue;

		if (jobno == killno)
			return(delete(entry->d_name, jobno, force, interact));
	}

	if (!force)
		printf("atrm: %d: no such job number\n", killno);
	
	return(1);
}

/* -------------------------------------------------------------------------- */

find_all(name, force, interact, dp)

char *name;
int force;
int interact;
DIR *dp;

{
	int jobno;
	int rc;
	int tod;
	int uid;

	struct dirent *entry;
	struct stat statbuf;

	rc = 0;

	/* If the owner name is "-", then delete all jobs for this user. */
	/* If the user is root, delete all jobs. */

	if (strcmp(name, "-") == 0) {

		uid = geteuid();

		if (uid == 0)
			uid = -1;		/* A flag meaning all jobs match */
	}

	else {

		struct passwd *pw;

		/* Get the uid of the named owner */

		if ((pw = getpwnam(name)) == NULL) {

			fprintf(stderr, "atrm: can't locate %s in /etc/passwd: %s\n",
				name, sys_errlist[errno]);
			return(1);
		}

		uid = pw->pw_uid;
	}

	/* Scan the directory for all job files with this owner */

	while ((entry = readdir(dp)) != NULL) {

		if (sscanf(entry->d_name, "%d.%d", &tod, &jobno) != 2)
			continue;

		if (uid >= 0) {

			if (stat(entry->d_name, &statbuf) < 0) {

				perror(entry->d_name);
				continue;
			}

			if (statbuf.st_uid != uid)
				continue;
		}

		rc |= delete(entry->d_name, jobno, force, interact);
	}

	return(rc);
}
